@@ -4,15 +4,16 @@ mod common;
 mod period;
 mod trades;
 
+use std::rc::Rc;
+
 #[cfg(test)] use crate::brokers::Broker;
 #[cfg(test)] use crate::config::Config;
 use crate::core::GenericResult;
 #[cfg(test)] use crate::taxes::TaxRemapping;
-use crate::xls::SheetParser;
+use crate::xls::{XlsStatementParser, Section, SheetParser};
 
 #[cfg(test)] use super::{BrokerStatement, ReadingStrictness};
 use super::{BrokerStatementReader, PartialBrokerStatement};
-use super::xls::{XlsStatementParser, Section};
 
 use assets::AssetsParser;
 use cash_flow::CashFlowParser;
@@ -29,15 +30,16 @@ impl StatementReader {
 }
 
 impl BrokerStatementReader for StatementReader {
-    fn is_statement(&self, path: &str) -> GenericResult<bool> {
+    fn check(&mut self, path: &str) -> GenericResult<bool> {
         Ok(path.ends_with(".xls"))
     }
 
     fn read(&mut self, path: &str, _is_last: bool) -> GenericResult<PartialBrokerStatement> {
         let parser = Box::new(StatementSheetParser{});
+        let statement = PartialBrokerStatement::new_rc(true);
 
         XlsStatementParser::read(path, parser, vec![
-            Section::new("Период:").parser(Box::new(PeriodParser{})).required(),
+            Section::new("Период:").parser(PeriodParser::new(statement.clone())).required(),
 
             Section::new("1. Движение денежных средств").required(),
             Section::new("1.1. Движение денежных средств по совершенным сделкам:").required(),
@@ -49,15 +51,17 @@ impl BrokerStatementReader for StatementReader {
                 .alias("Задолженность перед Компанией на начало периода (Рубль):").required(),
             Section::new("Остаток денежных средств на конец периода (Рубль):")
                 .alias("Задолженность перед Компанией на конец периода (Рубль):").required(),
-            Section::new("Рубль").parser(Box::new(CashFlowParser{})),
+            Section::new("Рубль").parser(CashFlowParser::new(statement.clone())),
 
             Section::new("2.1. Сделки:"),
-            Section::new("Пай").parser(Box::new(TradesParser{})),
+            Section::new("Пай").parser(TradesParser::new(statement.clone())),
             Section::new("2.3. Незавершенные сделки"),
 
             Section::new("3. Активы:").required(),
-            Section::new("Вид актива").parser(Box::new(AssetsParser{})).required(),
-        ])
+            Section::new("Вид актива").parser(AssetsParser::new(statement.clone())).required(),
+        ])?;
+
+        Rc::try_unwrap(statement).ok().unwrap().into_inner().validate()
     }
 }
 
